// Windows stuff
#include "StdAfx.h"

// Defines / Structures / Variable stuff
#include "general.h"

// Variable declarations
#include "D3DFunc.h"

// DLL entry point
BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved ) { return TRUE; }

// Tells the DBP compiler to always include the Setup DLL. (Required for the plugin to function)
// NOTE: Not added to export table, undecorated form required.
// Doesn't look like we'll be needing this anymore
// UPDATE: Looks like we may need this after all
EXPORT char const * GetDependencyID(int iID)
{
	if (iID = 1)
		return "DBProSetupDebug.dll";
	else
		return NULL;
}

// Tell the DBP compiler how many DLLs we require.
// NOTE: Not added to export table, undecorated form required.
// Or this...
// UPDATE: And this
EXPORT int GetNumDependencies( void ) { return 1; }

// NOTE: Not added to export table, undecorated form required.
EXPORT void ReceiveCoreDataPtr( void *globPtr )
{
	g_pGlob = (GlobStruct *) globPtr;
	InitD3D();
}

// Grab functions from DBPs DLLs
void SetupFunctionPointers ( void )
{
	if (g_pGlob->g_Basic3Dmade)
	{
		CreateNewObject					= ( BISI ) GetProcAddress(g_pGlob->g_Basic3D, "?CreateNewObject@@YA_NHPADH@Z");
		GetObjectData					= ( OI ) GetProcAddress(g_pGlob->g_Basic3D, "?GetObjectA@@YAPAUsObject@@H@Z");
		SetupFVFData					= ( BODDD )GetProcAddress(g_pGlob->g_Basic3D, "?SetupMeshFVFData@@YA_NPAUsMesh@@KKK@Z");
		CalculateMeshBounds				= ( BO ) GetProcAddress(g_pGlob->g_Basic3D, "?CalculateMeshBounds@@YA_NPAUsMesh@@@Z");
		SetNewObjectFinalProperties		= ( VIF ) GetProcAddress(g_pGlob->g_Basic3D, "?SetNewObjectFinalProperties@@YA_NHM@Z");
		SetTexture						= ( VII ) GetProcAddress(g_pGlob->g_Basic3D, "?SetTexture@@YAXHH@Z");
		SetDetailTexture				= ( VII ) GetProcAddress(g_pGlob->g_Basic3D, "?SetDetailMap@@YAXHH@Z");
	}

	if (g_pGlob->g_Camera3Dmade)
		GetInternalData					= ( DD ) GetProcAddress(g_pGlob->g_Camera3D, "?GetInternalData@@YAPAXH@Z");
}

// Initialize the plugin
void InitD3D()
{
	// Grab path to setup DLL
	char cPath[_MAX_PATH];
	strcpy(cPath, g_pGlob->pEXEUnpackDirectory);
	strcat(cPath, "\\DBProSetupDebug.dll");

	// Grab device pointer
	HMODULE hLib = LoadLibrary(cPath);
	OV getD3DDevice = (OV) GetProcAddress(hLib, "?GetDirect3DDevice@@YAPAUIDirect3DDevice9@@XZ");
	g_pD3DDevice = getD3DDevice();
	FreeLibrary(hLib);

	// Grab device pointer the easy way...
	// Which now seems to be broken with U6B4
	// g_pGlob->pCurrentBitmapSurface->GetDevice(&g_pD3DDevice);

	// Vertex buffer for drawing alpha 2D boxes
	if (g_pVB_Box)
		SAFE_RELEASE(g_pVB_Box);

	g_pD3DDevice->CreateVertexBuffer(4 * sizeof(TransformedVertex), D3DUSAGE_DYNAMIC | D3DUSAGE_WRITEONLY, D3DFVF_XYZRHW | D3DFVF_DIFFUSE, D3DPOOL_DEFAULT, &g_pVB_Box, NULL);

	// Create sprite for 2D text
	if (g_pTextSprite != NULL)
		SAFE_RELEASE(g_pTextSprite);
	HRESULT hr = D3DXCreateSprite(g_pD3DDevice, &g_pTextSprite);

	// Default font
	SetFont(1, "Arial", 15, 0, 0, 0);

	// Grab Pro function pointers
	SetupFunctionPointers();

	// Default drawing colour
	g_Forecolour = D3DCOLOR_ARGB(255, 255, 255, 255);
}

// Set global drawing colour
void DrawColor(int nRed, int nGreen, int nBlue, int nAlpha)
{
	g_Forecolour = D3DCOLOR_ARGB(nAlpha, nRed, nGreen, nBlue);
}

// Return colour with alpha
DWORD D3DRGBA(int iR, int iG, int iB, int iA)
{
	return D3DCOLOR_ARGB(iA, iR, iG, iB);
}

// Turn a DX Mesh into a Pro object
// TODO: Go over this
void MakeObjectFromDXMesh( int iID,ID3DXMesh *ppMesh )
{
    HRESULT  hr;
    int   iFinalMeshCount   = 1;
    int   iMesh     = 0;
    sObject* pObject     = NULL;
    sMesh*  pMesh     = NULL;  // temporary mesh pointer
    sFrame*  pFrame     = NULL;  // temporary frame pointer
    sMesh**  ppMeshListFromObject = NULL;  // mesh list array
    DWORD  dwIndexCount   = 0;  // index count
    DWORD  dwVertexCount   = 0;  // vertex count
    BYTE*  pVertices    =NULL;
    DWORD  nVertSize;
    LPDIRECT3DINDEXBUFFER9 pMeshIndexBuffer;
    short *pIndices;

    CreateNewObject( iID, "3DText", iFinalMeshCount );

    pObject = GetObjectData ( iID );

    if ( !pObject )
    {
        // error
    }

    // now get a pointer to the first mesh and first frame, finally
    // create a temporary mesh list so we can access by array and not
    // through the child structure, we can't use the object mesh list
    // at this stage because it hasn't been created yet

    pMesh     = pObject->pFrame->pMesh;
    pFrame        = pObject->pFrame;
    ppMeshListFromObject = new sMesh* [ iFinalMeshCount ];

    if ( !ppMeshListFromObject )
    {
        // error
    }

    // store pointers to meshes in the mesh list

    for ( iMesh = 0; iMesh < iFinalMeshCount; iMesh++ )
    {
        // store mesh pointer

        ppMeshListFromObject [ iMesh ] = pMesh;

        // move temporary pointers to child frames

        if ( pFrame->pChild )
        {
            pFrame = pFrame->pChild;
            pMesh  = pFrame->pMesh;
        }
    }

    // go through all meshes in list

    for ( iMesh = 0; iMesh < iFinalMeshCount; iMesh++ )
    {
        // get index and vertex count of the block

        dwIndexCount=ppMesh->GetNumFaces() * 3;
        dwVertexCount = ppMesh->GetNumVertices();

        // set up the new mesh

        SetupFVFData ( ppMeshListFromObject [ iMesh ], D3DFVF_DBPMESH, dwVertexCount, dwIndexCount );

        sVertex* pVertex = ( sVertex* ) ppMeshListFromObject [ iMesh ]->pVertexData;

        hr=ppMesh->LockVertexBuffer(D3DLOCK_READONLY, (LPVOID*)&pVertices);
        nVertSize=D3DXGetFVFVertexSize(ppMesh->GetFVF());

        //Get pointer to Index buffer and lock the index buffer in prder to access it

        ppMesh->GetIndexBuffer(&pMeshIndexBuffer);
        pMeshIndexBuffer->Lock(0, 3 * ppMesh->GetNumFaces() * sizeof(short), (void **)&pIndices, D3DLOCK_READONLY);

        // copy vertices from block into our new mesh

        for ( int iVertex = 0; iVertex < (int)dwVertexCount; iVertex++ )
        {
            sVertex p = *(sVertex*)(pVertices + iVertex*nVertSize);
            pVertex [ iVertex ].x         = p.x;
            pVertex [ iVertex ].y         = p.y;
            pVertex [ iVertex ].z         = p.z;

            pVertex [ iVertex ].nx         = p.nx;
            pVertex [ iVertex ].ny         = p.ny;
            pVertex [ iVertex ].nz         = p.nz;

            pVertex [ iVertex ].tu        = p.tu;
            pVertex [ iVertex ].tv        = p.tv;
        }
        ppMesh->UnlockVertexBuffer();

        // copy indices from block into mesh

        for ( int iIndex = 0; iIndex < (int)dwIndexCount; iIndex++ )
            ppMeshListFromObject [ iMesh ]->pIndices [ iIndex ] = pIndices[iIndex];

        pMeshIndexBuffer->Unlock();

        // set up mesh properties, no lights as we use diffuse, triangle list for primitive type, store
        // the vertex count and finally the primitive count

        ppMeshListFromObject [ iMesh ]->bLight           = true;
        ppMeshListFromObject [ iMesh ]->iPrimitiveType   = D3DPT_TRIANGLELIST;
        ppMeshListFromObject [ iMesh ]->iDrawVertexCount = ppMeshListFromObject [ iMesh ]->dwVertexCount;
        ppMeshListFromObject [ iMesh ]->iDrawPrimitives  = ppMesh->GetNumFaces ( );
        ppMeshListFromObject [ iMesh ]->iMeshType   = 1;

        // calculate the mesh bounds

        CalculateMeshBounds ( ppMeshListFromObject [ iMesh ] );
    }

    // set the final properties of the object and apply a blank texture

    SetNewObjectFinalProperties ( iID, 0.0f );
    SetTexture                  ( iID, 0 );

    // safely delete the temporary mesh list

    SAFE_DELETE_ARRAY ( ppMeshListFromObject );
    SAFE_RELEASE ( ppMesh );
}