// Windows stuff
#include "StdAfx.h"

// Defines / Structures / Variable stuff
#include "general.h"

// Batch 2D dot stuff
TransformedVertex *g_BatchDot2DList = NULL;
int iBatchDot2DPos = 0;		// Position in batch
int iBatchDot2DMax = 0;		// Size of batch

// Batch 2D line stuff
TransformedVertex *g_BatchLine2DList = NULL;
int iBatchLine2DPos = 0;	// Position in batch
int iBatchLine2DMax = 0;	// Batch size

// 2D box with colours per vertex.
void GradientBox(int x1, int y1, int x2, int y2, DWORD dwColourA, DWORD dwColourB, DWORD dwColourC, DWORD dwColourD)
{
	TransformedVertex *pVertices = NULL;

	g_pTS_Box[0].x = (float)x1;
    g_pTS_Box[0].y = (float)y1;
    g_pTS_Box[0].color = dwColourA;

    g_pTS_Box[1].x = (float)x2;
    g_pTS_Box[1].y = (float)y1;
    g_pTS_Box[1].color = dwColourB;

    g_pTS_Box[2].x = (float)x1;
    g_pTS_Box[2].y = (float)y2;
    g_pTS_Box[2].color = dwColourC;

    g_pTS_Box[3].x = (float)x2;
    g_pTS_Box[3].y = (float)y2;
    g_pTS_Box[3].color = dwColourD;

	g_pD3DDevice->SetTexture(0, NULL);
    g_pD3DDevice->SetTexture(1, NULL);
    g_pD3DDevice->SetVertexShader(NULL);
    g_pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
    g_pD3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
    g_pD3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
    g_pD3DDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_FALSE);

    g_pVB_Box->Lock(0, sizeof(g_pTS_Box), (void**)&pVertices, D3DLOCK_DISCARD);
    memcpy(pVertices, g_pTS_Box, sizeof(g_pTS_Box));
    g_pVB_Box->Unlock();

    g_pD3DDevice->SetStreamSource(0, g_pVB_Box, 0, sizeof(TransformedVertex));
    g_pD3DDevice->SetFVF(D3DFVF_2D_VERTEX);
    g_pD3DDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);
}

// Normal 2D box
void Box(int x1, int y1, int x2, int y2)
{
	GradientBox(x1, y1, x2, y2, g_Forecolour, g_Forecolour, g_Forecolour, g_Forecolour);
}

// 2D box with overriding colour
void ColourBox(int x1, int y1, int x2, int y2, DWORD dwColour)
{
	GradientBox(x1, y1, x2, y2, dwColour, dwColour, dwColour, dwColour);
}

// Unfilled polygon
void PolygonUnfilled(int iX, int iY, float fRadiusX, float fRadiusY, int iVertexCount, DWORD dwColour)
{
	// Minimum of 3 vertex, else we're drawing a dot / line.
	if (iVertexCount < 3)
		return;

	// Line list, batch draw everything
	TransformedVertex *g_lineList;
	g_lineList = new (nothrow) TransformedVertex [iVertexCount+1];

	// Check memory was allocated successfully
	if (!g_lineList)
		return;

	// Position data
	int iAX = 0, iAY = 0, iBX = 0, iBY = 0;

	// Angle of current vert.
	float fAngle;
	for (int i = 0; i < iVertexCount; i++)
	{
		iBX = iAX; iBY = iAY;
		
		// 6.28318 = PI * 2
		fAngle = i / (float)iVertexCount * 6.283185307179586476925286766559f;
		iAX = (int)(iX + (sin(fAngle) * fRadiusX));
		iAY = (int)(iY + (cos(fAngle) * fRadiusY));

		
		if (i == 0)
		{
			g_lineList[iVertexCount].x = (float)iAX;
			g_lineList[iVertexCount].y = (float)iAY;
			g_lineList[iVertexCount].z = 1.0f;
			g_lineList[iVertexCount].w = 1.0f;
			g_lineList[iVertexCount].color = dwColour;
		}

		g_lineList[i].x = (float)iAX;
		g_lineList[i].y = (float)iAY;
		g_lineList[i].z = 1.0f;
		g_lineList[i].w = 1.0f;
		g_lineList[i].color = dwColour;
	}

	// Draw
	g_pD3DDevice->SetTexture(0, NULL);
    g_pD3DDevice->SetTexture(1, NULL);
    g_pD3DDevice->SetVertexShader(NULL);
    g_pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
    g_pD3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
    g_pD3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
    g_pD3DDevice->SetFVF(D3DFVF_2D_VERTEX);
    g_pD3DDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_FALSE);

    g_pD3DDevice->DrawPrimitiveUP(D3DPT_LINESTRIP, iVertexCount, g_lineList, sizeof(TransformedVertex));

	// Delete allocated memory
	delete[] g_lineList;
}

// Filled polygon
void PolygonFilled(int iX, int iY, float fRadiusX, float fRadiusY, int iVertexCount, DWORD dwColour, int iFilled)
{
	// Minimum of 3 vertex, else we're drawing a dot / line.
	if (iVertexCount < 3)
		return;

	// Line list, batch draw everything
	TransformedVertex *g_lineList;
	g_lineList = new (nothrow) TransformedVertex [iVertexCount + 2];

	// Check memory was allocated successfully
	if (!g_lineList)
		return;

	// Position data
	int iAX = 0, iAY = 0, iBX = 0, iBY = 0;

	// Angle of current vert.
	float fAngle;

	g_lineList[0].x = (float)iX;
	g_lineList[0].y = (float)iY;
	g_lineList[0].z = 1.0f;
	g_lineList[0].w = 1.0f;
	g_lineList[0].color = dwColour;

	for (int i = 0; i < iVertexCount; i++)
	{
		iBX = iAX; iBY = iAY;
		
		// 6.28318 = PI * 2
		fAngle = i / (float)iVertexCount * 6.28f;
		iAX = (int)(iX + (sin(fAngle) * fRadiusX));
		iAY = (int)(iY + (cos(fAngle) * fRadiusY));

		if (i == 0)
		{
			g_lineList[iVertexCount + 1].x = (float)iAX;
			g_lineList[iVertexCount + 1].y = (float)iAY;
			g_lineList[iVertexCount + 1].z = 1.0f;
			g_lineList[iVertexCount + 1].w = 1.0f;
			g_lineList[iVertexCount + 1].color = dwColour;
		}
		
		g_lineList[i + 1].x = (float)iAX;
		g_lineList[i + 1].y = (float)iAY;
		g_lineList[i + 1].z = 1.0f;
		g_lineList[i + 1].w = 1.0f;
		g_lineList[i + 1].color = dwColour;
	}

	// Draw
	g_pD3DDevice->SetTexture(0, NULL);
    g_pD3DDevice->SetTexture(1, NULL);
    g_pD3DDevice->SetVertexShader(NULL);
    g_pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
    g_pD3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
    g_pD3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
    g_pD3DDevice->SetFVF(D3DFVF_2D_VERTEX);
    g_pD3DDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_FALSE);

    g_pD3DDevice->DrawPrimitiveUP(D3DPT_TRIANGLEFAN, iVertexCount, g_lineList, sizeof(TransformedVertex));

	// Delete allocated memory
	delete[] g_lineList;
}

// Standard 2D circle
void Circle(int iX, int iY, float fRadius, int iFilled)
{
	if (iFilled == 0)
		PolygonUnfilled(iX, iY, fRadius, fRadius, 128, g_Forecolour);
	else
		PolygonFilled(iX, iY, fRadius, fRadius, 128, g_Forecolour, 0);
}

// Standard 2D circle with overriding colour
void ColourCircle(int iX, int iY, float fRadius, int iFilled, DWORD dwColour)
{
	if (iFilled == 0)
		PolygonUnfilled(iX, iY, fRadius, fRadius, 128, dwColour);
	else
		PolygonFilled(iX, iY, fRadius, fRadius, 128, dwColour, 0);
}

// Standard 2D ellipse (Misspelled to avoid conflicts with GDI, string table is correct)
void Elipse(int iX, int iY, float fRadiusX, float fRadiusY, int iFilled)
{
	if (iFilled==0)
		PolygonUnfilled(iX, iY, fRadiusX, fRadiusY, 128, g_Forecolour);
	else
		PolygonFilled(iX, iY, fRadiusX, fRadiusY, 128, g_Forecolour, 0);
}

// Standard 2D ellipse with overriding colour (Misspelled to avoid conflicts with GDI, string table is correct)
void ColourElipse(int iX, int iY, float fRadiusX, float fRadiusY, int iFilled, DWORD dwColour)
{
	if (iFilled==0)
		PolygonUnfilled(iX, iY, fRadiusX, fRadiusY, 128, dwColour);
	else
		PolygonFilled(iX, iY, fRadiusX, fRadiusY, 128, dwColour, 0);
}

// 2D Polygon
void D3DPolygon(int iX, int iY, float fRadius, int iVertexCount, int iFilled)
{
	if (iFilled==0)
		PolygonUnfilled(iX, iY, fRadius, fRadius, iVertexCount, g_Forecolour);
	else
		PolygonFilled(iX, iY, fRadius, fRadius, iVertexCount, g_Forecolour, 0);
}

// 2D Polygon with overriding colour
void ColourPolygon(int iX, int iY, float fRadius, int iVertexCount, int iFilled, DWORD dwColour)
{
	if (iFilled==0)
		PolygonUnfilled(iX, iY, fRadius, fRadius, iVertexCount, dwColour);
	else
		PolygonFilled(iX, iY, fRadius, fRadius, iVertexCount, dwColour, 0);
}

// 2D dot with overriding colour
void ColourDot(int x1, int y1, DWORD dwColour)
{
	TransformedVertex	g_VertList[1] = {(float)x1, (float)y1, 1.0f, 1.0f, dwColour};

	g_pD3DDevice->SetTexture(0, NULL);
    g_pD3DDevice->SetTexture(1, NULL);
    g_pD3DDevice->SetVertexShader(NULL);
    g_pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
    g_pD3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
    g_pD3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
    g_pD3DDevice->SetFVF(D3DFVF_2D_VERTEX);
    g_pD3DDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_FALSE);

    g_pD3DDevice->DrawPrimitiveUP(D3DPT_POINTLIST, 1, g_VertList, sizeof(TransformedVertex));
}

// Standard 2D dot
void Dot(int x1, int y1)
{
	ColourDot(x1, y1, g_Forecolour);
}

// 2D line with gradient colour
void GradientLine(int iX1, int iY1, int iX2, int iY2, DWORD dwColour1, DWORD dwColour2)
{
	TransformedVertex g_LineList[2] =	{
											{ (float)iX1, (float)iY1, 1.0f, 1.0f, dwColour1 },
											{ (float)iX2, (float)iY2, 1.0f, 1.0f, dwColour2 }
										};

	g_pD3DDevice->SetTexture(0, NULL);
    g_pD3DDevice->SetTexture(1, NULL);
    g_pD3DDevice->SetVertexShader(NULL);
    g_pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
    g_pD3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
    g_pD3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
    g_pD3DDevice->SetFVF(D3DFVF_2D_VERTEX);
    g_pD3DDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_FALSE);

	g_pD3DDevice->DrawPrimitiveUP(D3DPT_LINELIST, 1, g_LineList, sizeof(TransformedVertex));
}

// 2D line with colour override
void ColourLine(int x1, int y1, int x2, int y2, DWORD dwColour)
{
	GradientLine(x1, y1, x2, y2, dwColour, dwColour);
}

// Standard 2D line
void Line(int x1, int y1, int x2, int y2)
{
	GradientLine(x1, y1, x2, y2, g_Forecolour, g_Forecolour);
}

// Initialize 2D dot batch
void SetBatchSizeDot2D(int iSize)
{
	// If we already have a batch, kill it
	if (g_BatchDot2DList)
		delete[] g_BatchDot2DList;

	// Create new batch
	g_BatchDot2DList = new (nothrow) TransformedVertex[iSize];

	if (g_BatchDot2DList)
	{
		// New batch successful
		iBatchDot2DPos = 0;
		iBatchDot2DMax = iSize;
	} else {
		// New batch failed
		iBatchDot2DPos = 0;
		iBatchDot2DMax = 0;
	}
}

// Add 2D dot to batch
void AddBatchDot2D(int iX, int iY)
{
	// If we haven't reached the batch limit
	if (iBatchDot2DPos < iBatchDot2DMax)
	{
		// Add dot
		g_BatchDot2DList[iBatchDot2DPos].x = (float)iX;
		g_BatchDot2DList[iBatchDot2DPos].y = (float)iY;
		g_BatchDot2DList[iBatchDot2DPos].z = 1.0f;
		g_BatchDot2DList[iBatchDot2DPos].w = 1.0f;
		g_BatchDot2DList[iBatchDot2DPos].color = g_Forecolour;

		iBatchDot2DPos++;
	}
}

// Add 2D dot with overriding colour to batch
void ColourAddBatchDot2D(int iX, int iY, DWORD dwColour)
{
	// If we haven't reached the batch limit
	if (iBatchDot2DPos < iBatchDot2DMax)
	{
		// Add dot
		g_BatchDot2DList[iBatchDot2DPos].x = (float)iX;
		g_BatchDot2DList[iBatchDot2DPos].y = (float)iY;
		g_BatchDot2DList[iBatchDot2DPos].z = 1.0f;
		g_BatchDot2DList[iBatchDot2DPos].w = 1.0f;
		g_BatchDot2DList[iBatchDot2DPos].color = dwColour;

		iBatchDot2DPos++;
	}
}

// Draw 2D dot batch
void BatchDrawDot2D(int iClear)
{
	g_pD3DDevice->SetTexture(0, NULL);
    g_pD3DDevice->SetTexture(1, NULL);
    g_pD3DDevice->SetVertexShader(NULL);
    g_pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
    g_pD3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
    g_pD3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
    g_pD3DDevice->SetFVF(D3DFVF_2D_VERTEX);
    g_pD3DDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_FALSE);

    g_pD3DDevice->DrawPrimitiveUP(D3DPT_POINTLIST, iBatchDot2DPos, g_BatchDot2DList, sizeof(TransformedVertex));

	// Should we delete the batch
	if (iClear != 0)
	{
		delete[] g_BatchDot2DList;
		g_BatchDot2DList = NULL;
		iBatchDot2DMax = 0;
	}

	iBatchDot2DPos = 0;
}

// Initialize 2D line batch
void SetBatchSizeLine2D(int iSize)
{
	// If we have a batch, kill it
	if (g_BatchLine2DList)
		delete[] g_BatchLine2DList;

	// Create new batch
	g_BatchLine2DList = new (nothrow) TransformedVertex[iSize * 2];

	if (g_BatchLine2DList)
	{
		// Batch creation succeeded
		iBatchLine2DPos = 0;
		iBatchLine2DMax = iSize;
	} else {
		// Batch creation failed: Make sure we can never add to it
		iBatchLine2DPos = 0;
		iBatchLine2DMax = 0;
	}
}

// Add 2D line to batch
void AddBatchLine2D(int iX1, int iY1, int iX2, int iY2)
{
	// If we haven't reached the batch limit
	if (iBatchLine2DPos < iBatchLine2DMax)
	{
		g_BatchLine2DList[(iBatchLine2DPos * 2)].x = (float)iX1;
		g_BatchLine2DList[(iBatchLine2DPos * 2)].y = (float)iY1;
		g_BatchLine2DList[(iBatchLine2DPos * 2)].z = 1.0f;
		g_BatchLine2DList[(iBatchLine2DPos * 2)].w = 1.0f;
		g_BatchLine2DList[(iBatchLine2DPos * 2)].color = g_Forecolour;

		g_BatchLine2DList[(iBatchLine2DPos * 2) + 1].x = (float)iX2;
		g_BatchLine2DList[(iBatchLine2DPos * 2) + 1].y = (float)iY2;
		g_BatchLine2DList[(iBatchLine2DPos * 2) + 1].z = 1.0f;
		g_BatchLine2DList[(iBatchLine2DPos * 2) + 1].w = 1.0f;
		g_BatchLine2DList[(iBatchLine2DPos * 2) + 1].color = g_Forecolour;

		iBatchLine2DPos++;
	}
}

// Add 2D line to batch with overriding colour
void ColourAddBatchLine2D(int iX1, int iY1, int iX2, int iY2, DWORD dwColour)
{
	// If we haven't reached the batch limit
	if (iBatchLine2DPos < iBatchLine2DMax)
	{
		g_BatchLine2DList[(iBatchLine2DPos * 2)].x = (float)iX1;
		g_BatchLine2DList[(iBatchLine2DPos * 2)].y = (float)iY1;
		g_BatchLine2DList[(iBatchLine2DPos * 2)].z = 1.0f;
		g_BatchLine2DList[(iBatchLine2DPos * 2)].w = 1.0f;
		g_BatchLine2DList[(iBatchLine2DPos * 2)].color = dwColour;

		g_BatchLine2DList[(iBatchLine2DPos * 2) + 1].x = (float)iX2;
		g_BatchLine2DList[(iBatchLine2DPos * 2) + 1].y = (float)iY2;
		g_BatchLine2DList[(iBatchLine2DPos * 2) + 1].z = 1.0f;
		g_BatchLine2DList[(iBatchLine2DPos * 2) + 1].w = 1.0f;
		g_BatchLine2DList[(iBatchLine2DPos * 2) + 1].color = dwColour;

		iBatchLine2DPos++;
	}
}

// Add 2D line to batch with gradient colour
void GradientAddBatchLine2D(int iX1, int iY1, int iX2, int iY2, DWORD dwColour1, DWORD dwColour2)
{
	// If we haven't reached the batch limit
	if (iBatchLine2DPos < iBatchLine2DMax)
	{
		g_BatchLine2DList[(iBatchLine2DPos * 2)].x = (float)iX1;
		g_BatchLine2DList[(iBatchLine2DPos * 2)].y = (float)iY1;
		g_BatchLine2DList[(iBatchLine2DPos * 2)].z = 1.0f;
		g_BatchLine2DList[(iBatchLine2DPos * 2)].w = 1.0f;
		g_BatchLine2DList[(iBatchLine2DPos * 2)].color = dwColour1;

		g_BatchLine2DList[(iBatchLine2DPos * 2) + 1].x = (float)iX2;
		g_BatchLine2DList[(iBatchLine2DPos * 2) + 1].y = (float)iY2;
		g_BatchLine2DList[(iBatchLine2DPos * 2) + 1].z = 1.0f;
		g_BatchLine2DList[(iBatchLine2DPos * 2) + 1].w = 1.0f;
		g_BatchLine2DList[(iBatchLine2DPos * 2) + 1].color = dwColour2;

		iBatchLine2DPos++;
	}
}

// Draw 2D line batch
void BatchDrawLine2D(int iClear)
{
	g_pD3DDevice->SetTexture(0, NULL);
    g_pD3DDevice->SetTexture(1, NULL);
    g_pD3DDevice->SetVertexShader(NULL);
    g_pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
    g_pD3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
    g_pD3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
    g_pD3DDevice->SetFVF(D3DFVF_2D_VERTEX);
    g_pD3DDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_FALSE);

	g_pD3DDevice->DrawPrimitiveUP(D3DPT_LINELIST, iBatchLine2DPos, g_BatchLine2DList, sizeof(TransformedVertex));

	if (iClear)
	{
		delete[] g_BatchLine2DList;
		g_BatchLine2DList = NULL;
		iBatchLine2DMax = 0;
	}

	iBatchLine2DPos = 0;
}