// Windows stuff
#include "StdAfx.h"

// Defines / Structures / Variable stuff
#include "general.h"

// Batch 3D dot stuff
Vertex *g_BatchDot3DList = NULL;
int iBatchDot3DPos = 0;				// Current position in batch
int iBatchDot3DMax = 0;				// Batch size
float fBatchDot3DSize = 1.0f;		// Batch dot 3D size

// Batch 3D line stuff
Vertex *g_BatchLine3DList = NULL;
int iBatchLine3DPos = 0;			// Current position in batch
int iBatchLine3DMax = 0;			// Batch size

// Standard dot in 3D space with camera parameter
void Dot3DCamera(int iCamera, float fX, float fY, float fZ, float fSize, DWORD dwColour, int iZEnable)
{
	Vertex g_DotList[] = {fX, fY, fZ, dwColour};

	DWORD dwCameraInternalData = GetInternalData((DWORD)iCamera);
	if (!dwCameraInternalData)
	{
		// Invalid camera
		*(DWORD*)g_pGlob->g_pErrorHandlerRef = (DWORD)7203;
		return;
	}

	g_pViewPort.X = *(int*)(dwCameraInternalData+200);
    g_pViewPort.Y = *(int*)(dwCameraInternalData+204);
    g_pViewPort.Width = *(int*)(dwCameraInternalData+208);
    g_pViewPort.Height = *(int*)(dwCameraInternalData+212);
	// TODO: Should these be CameraNear/Far?
    g_pViewPort.MinZ = 0.0f;
    g_pViewPort.MaxZ = 1.0f;

    D3DXMATRIX matWorld;
	D3DXMatrixIdentity(&matWorld);

	D3DXMATRIX matProj = *(D3DXMATRIX*)(dwCameraInternalData);
    D3DXMATRIX matView = *(D3DXMATRIX*)(dwCameraInternalData+64);

    g_pD3DDevice->SetTransform(D3DTS_WORLD, &matWorld);
    g_pD3DDevice->SetTransform(D3DTS_VIEW, &matView);
    g_pD3DDevice->SetTransform(D3DTS_PROJECTION, &matProj);
    g_pD3DDevice->SetTexture(0, NULL);
    g_pD3DDevice->SetTexture(1, NULL);
    g_pD3DDevice->SetVertexShader(NULL);
	g_pD3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
    g_pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
    g_pD3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
    g_pD3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
    g_pD3DDevice->SetRenderState(D3DRS_ZENABLE, iZEnable);
	g_pD3DDevice->SetRenderState(D3DRS_POINTSIZE, *((DWORD*)&fSize));
    g_pD3DDevice->SetViewport(&g_pViewPort);
    g_pD3DDevice->SetFVF(D3DFVF_MY_VERTEX);
    g_pD3DDevice->DrawPrimitiveUP(D3DPT_POINTLIST, 1, g_DotList, sizeof(Vertex));
}

// Standard dot in 3D space with overriding colour and Z state
void Dot3DCZ(float fX, float fY, float fZ, float fSize, DWORD dwColour, int iZEnable)
{
	Dot3DCamera(0, fX, fY, fZ, fSize, dwColour, iZEnable);
}

// Standard dot in 3D space
void Dot3D(float fX, float fY, float fZ, float fSize)
{
	Dot3DCamera(0, fX, fY, fZ, fSize, g_Forecolour, 1);
}

// Standard line in 3D space with camera parameter
void Line3DCamera(int iCamera, float fX1, float fY1, float fZ1, float fX2, float fY2, float fZ2, DWORD dwColour1, DWORD dwColour2, int iZEnable)
{
	Vertex g_lineList[] =	{
								{fX1, fY1, fZ1, dwColour1},
								{fX2, fY2, fZ2, dwColour2}
							};

	DWORD dwCameraInternalData = GetInternalData((DWORD)iCamera);
	if (!dwCameraInternalData)
	{
		// Invalid camera
		*(DWORD*)g_pGlob->g_pErrorHandlerRef = (DWORD)7203;
		return;
	}

	g_pViewPort.X = *(int*)(dwCameraInternalData+200);
    g_pViewPort.Y = *(int*)(dwCameraInternalData+204);
    g_pViewPort.Width = *(int*)(dwCameraInternalData+208);
    g_pViewPort.Height = *(int*)(dwCameraInternalData+212);
	// TODO: Should these be CameraNear/Far?
    g_pViewPort.MinZ = 0.0f;
    g_pViewPort.MaxZ = 1.0f;

	D3DXMATRIX matWorld;
	D3DXMatrixIdentity(&matWorld);

	D3DXMATRIX matProj = *(D3DXMATRIX*)(dwCameraInternalData);
    D3DXMATRIX matView = *(D3DXMATRIX*)(dwCameraInternalData+64);

	g_pD3DDevice->SetTransform(D3DTS_WORLD, &matWorld);
    g_pD3DDevice->SetTransform(D3DTS_VIEW, &matView);
    g_pD3DDevice->SetTransform(D3DTS_PROJECTION, &matProj);
    g_pD3DDevice->SetTexture(0, NULL);
    g_pD3DDevice->SetTexture(1, NULL);
    g_pD3DDevice->SetVertexShader(NULL);
	g_pD3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
    g_pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
    g_pD3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
    g_pD3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
    g_pD3DDevice->SetRenderState(D3DRS_ZENABLE, iZEnable);
    g_pD3DDevice->SetViewport(&g_pViewPort);
    g_pD3DDevice->SetFVF(D3DFVF_MY_VERTEX);

	g_pD3DDevice->DrawPrimitiveUP(D3DPT_LINELIST, 1, g_lineList, sizeof(Vertex));
}

// Standard line in 3D space with overriding gradient colour / ZEnable
void Line3DGCZ(float fX1, float fY1, float fZ1, float fX2, float fY2, float fZ2, DWORD dwColour1, DWORD dwColour2, int iZEnable)
{
	Line3DCamera(0, fX1, fY1, fZ1, fX2, fY2, fZ2, dwColour1, dwColour2, iZEnable);
}

// Standard line in 3D space with overriding colour / ZEnable
void Line3DCZ(float fX1, float fY1, float fZ1, float fX2, float fY2, float fZ2, DWORD dwColour, int iZEnable)
{
	Line3DCamera(0, fX1, fY1, fZ1, fX2, fY2, fZ2, dwColour, dwColour, iZEnable);
}

// Standard line in 3D space
void Line3D(float fX1, float fY1, float fZ1, float fX2, float fY2, float fZ2)
{
	Line3DCamera(0, fX1, fY1, fZ1, fX2, fY2, fZ2, g_Forecolour, g_Forecolour, 1);
}

// Based on code by originally written by Philip (with amendments by Danny Gregory)
// Based on original polygon code
void Polygon3D(int iCamera, float fRadiusX, float fRadiusY, float fX, float fY, float fZ, float fNX, float fNY, float fNZ, int iVertexCount, DWORD dwColour, int iZEnable)
{
	// Minimum 3 verts
	if (iVertexCount < 3)
		return;

	// Line list, batch draw everything
	Vertex *g_lineList;
	g_lineList = new (nothrow) Vertex[iVertexCount + 1];

	if (!g_lineList)
		return;

	//Get Camera Info
	DWORD CameraInternalData = GetInternalData((DWORD)iCamera);
	if ( CameraInternalData == 0)
	{
		*(DWORD*)g_pGlob->g_pErrorHandlerRef = (DWORD)7203;
		return;
	}

	// Camera stuff
	D3DXMATRIX ProjectionMatrix = *(D3DXMATRIX*)(CameraInternalData);
    D3DXMATRIX ViewMatrix = *(D3DXMATRIX*)(CameraInternalData+64);
	g_pViewPort.X = *(int*)(CameraInternalData+200);
    g_pViewPort.Y = *(int*)(CameraInternalData+204);
    g_pViewPort.Width = *(int*)(CameraInternalData+208);
    g_pViewPort.Height = *(int*)(CameraInternalData+212);
    g_pViewPort.MinZ = 0.0f;
    g_pViewPort.MaxZ = 1.0f;

	D3DXVECTOR3 vNormal, vCamera, vCross;
	D3DXMATRIX mA;
	D3DXMATRIX mTranslation;

	// vNormal = Normalised Vector for plane of circle
	vNormal = D3DXVECTOR3(fNX*-1, fNY*-1, fNZ);
	D3DXVec3Normalize(&vNormal, &vNormal);

	//vCamera = Normalised vector for untransformed camera
	vCamera = D3DXVECTOR3(0, 0, 1);

	//Get vector perpendicular to the camera and circle
	D3DXVec3Cross(&vCross, &vNormal, &vCamera);

	//Get angle between camera and circle
	float fDotProduct=acos(D3DXVec3Dot(&vNormal,&vCamera));

	//Now Create a rotation axis for transformation
	D3DXMatrixRotationAxis(&mA, &vCross, fDotProduct);

	//Create Translation Matrix to position center of circle
	D3DXMatrixTranslation(&mTranslation, fX, fY, fZ);
	D3DXMATRIX matWorld;

	//Multiply rotation and translation to get final Matrix
	matWorld=mA*mTranslation;

	//Draw circle at origin
	float iAX=0.0f, iAY=0.0f, iBX=0.0f, iBY=0.0f, fAngle=0.0f;

	for (int i = 0; i < iVertexCount; i++)
	{
		iBX = iAX; iBY = iAY;
		
		// 6.28318 = PI * 2
		fAngle = i / (float)iVertexCount * 6.283185307179586476925286766559f;
		iAX = ((sin(fAngle) * fRadiusX));
		iAY = ((cos(fAngle) * fRadiusY));

		
		if (i == 0)
		{
			g_lineList[iVertexCount].x = iAX;
			g_lineList[iVertexCount].y = iAY;
			g_lineList[iVertexCount].z = 0.0f;
			g_lineList[iVertexCount].color = dwColour;
		}

		g_lineList[i].x = iAX;
		g_lineList[i].y = iAY;
		g_lineList[i].z = 0.0f;
		g_lineList[i].color = dwColour;
	}

	// Translate lines and draw
    g_pD3DDevice->SetTransform(D3DTS_WORLD, &matWorld);
    g_pD3DDevice->SetTransform(D3DTS_VIEW, &ViewMatrix);
    g_pD3DDevice->SetTransform(D3DTS_PROJECTION, &ProjectionMatrix);
    g_pD3DDevice->SetTexture(0, NULL);
    g_pD3DDevice->SetTexture(1, NULL);
    g_pD3DDevice->SetVertexShader(NULL);
	g_pD3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	g_pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
    g_pD3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
    g_pD3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
    g_pD3DDevice->SetRenderState(D3DRS_ZENABLE, iZEnable);
    g_pD3DDevice->SetViewport(&g_pViewPort);
    g_pD3DDevice->SetFVF(D3DFVF_MY_VERTEX);
	g_pD3DDevice->DrawPrimitiveUP(D3DPT_LINESTRIP, iVertexCount, g_lineList, sizeof(Vertex));

	// Free memory
	delete[] g_lineList;
}

// Standard circle in 3D space
void Circle3D(float fRadius, float fX, float fY, float fZ, float fNX, float fNY, float fNZ)
{
	Polygon3D(0, fRadius, fRadius, fX, fY, fZ, fNX, fNY, fNZ, 128, g_Forecolour, 1);
}

// Standard circle in 3D space with colour and Z enable
void Circle3DCZ(float fRadius, float fX, float fY, float fZ, float fNX, float fNY, float fNZ, DWORD dwColour, int iZEnable)
{
	Polygon3D(0, fRadius, fRadius, fX, fY, fZ, fNX, fNY, fNZ, 128, dwColour, iZEnable);
}

// Standard circle in 3D space inc. camera
void Circle3DCamera(int iCamera, float fRadius, float fX, float fY, float fZ, float fNX, float fNY, float fNZ, DWORD dwColour, int iZEnable)
{
	Polygon3D(iCamera, fRadius, fRadius, fX, fY, fZ, fNX, fNY, fNZ, 128, dwColour, iZEnable);
}

// Standard ellipse in 3D space
void Ellipse3D(float fRadiusX, float fRadiusY, float fX, float fY, float fZ, float fNX, float fNY, float fNZ)
{
	Polygon3D(0, fRadiusX, fRadiusY, fX, fY, fZ, fNX, fNY, fNZ, 128, g_Forecolour, 1);
}

// Standard ellipse in 3D space with colour and Z enable
void Ellipse3DCZ(float fRadiusX, float fRadiusY, float fX, float fY, float fZ, float fNX, float fNY, float fNZ, DWORD dwColour, int iZEnable)
{
	Polygon3D(0, fRadiusX, fRadiusY, fX, fY, fZ, fNX, fNY, fNZ, 128, dwColour, iZEnable);
}

// Standard ellipse in 3D space inc. camera
void Ellipse3DCamera(int iCamera, float fRadiusX, float fRadiusY, float fX, float fY, float fZ, float fNX, float fNY, float fNZ, DWORD dwColour, int iZEnable)
{
	Polygon3D(iCamera, fRadiusX, fRadiusY, fX, fY, fZ, fNX, fNY, fNZ, 128, dwColour, iZEnable);
}

// Initialize 3D dot batch
void SetBatchSizeDot3D(int iSize)
{
	// If we have a batch, kill it
	if (g_BatchDot3DList)
		delete[] g_BatchDot3DList;

	// Create new batch
	g_BatchDot3DList = new (nothrow) Vertex[iSize];

	if (g_BatchDot3DList)
	{
		// Batch creation succeeded
		iBatchDot3DPos = 0;
		iBatchDot3DMax = iSize;
	} else {
		// Batch creation failed: Make sure we can never add to it
		iBatchDot3DPos = 0;
		iBatchDot3DMax = 0;
	}
}

// Set dot size for batch
void SetBatchDot3DSize(float fSize)
{
	fBatchDot3DSize = fSize;
}

// Add 3D dot to batch
void AddBatchDot3D(float fX, float fY, float fZ)
{
	// If we haven't reached the batch limit
	if (iBatchDot3DPos < iBatchDot3DMax)
	{
		// Add dot
		g_BatchDot3DList[iBatchDot3DPos].x = fX;
		g_BatchDot3DList[iBatchDot3DPos].y = fY;
		g_BatchDot3DList[iBatchDot3DPos].z = fZ;
		g_BatchDot3DList[iBatchDot3DPos].color = g_Forecolour;

		iBatchDot3DPos++;
	}
}

// Add 3D dot to batch with overriding colour
void ColourAddBatchDot3D(float fX, float fY, float fZ, DWORD dwColour)
{
	// If we haven't reached the batch limit
	if (iBatchDot3DPos < iBatchDot3DMax)
	{
		// Add dot
		g_BatchDot3DList[iBatchDot3DPos].x = fX;
		g_BatchDot3DList[iBatchDot3DPos].y = fY;
		g_BatchDot3DList[iBatchDot3DPos].z = fZ;
		g_BatchDot3DList[iBatchDot3DPos].color = dwColour;

		iBatchDot3DPos++;
	}
}

// Draw 3D dot batch
void BatchDrawDot3D(int iCamera, int iZEnable, int iClear)
{
	// Get Camera Info
	DWORD dwCameraInternalData = GetInternalData((DWORD)iCamera);
	if (!dwCameraInternalData)
	{
		// Invalid camera
		*(DWORD*)g_pGlob->g_pErrorHandlerRef = (DWORD)7203;
		return;
	}

	g_pViewPort.X = *(int*)(dwCameraInternalData+200);
    g_pViewPort.Y = *(int*)(dwCameraInternalData+204);
    g_pViewPort.Width = *(int*)(dwCameraInternalData+208);
    g_pViewPort.Height = *(int*)(dwCameraInternalData+212);
	// TODO: Should these be CameraNear/Far?
	// CameraNear = *(float*)(dwCameraInternalData+344);
    // CameraFar = *(float*)(dwCameraInternalData+348);
    g_pViewPort.MinZ = 0.0f;
    g_pViewPort.MaxZ = 1.0f;

    D3DXMATRIX matWorld;
	D3DXMatrixIdentity(&matWorld);

	D3DXMATRIX matProj = *(D3DXMATRIX*)(dwCameraInternalData);
    D3DXMATRIX matView = *(D3DXMATRIX*)(dwCameraInternalData+64);

    g_pD3DDevice->SetTransform(D3DTS_WORLD, &matWorld);
    g_pD3DDevice->SetTransform(D3DTS_VIEW, &matView);
    g_pD3DDevice->SetTransform(D3DTS_PROJECTION, &matProj);
    g_pD3DDevice->SetTexture(0, NULL);
    g_pD3DDevice->SetTexture(1, NULL);
    g_pD3DDevice->SetVertexShader(NULL);
	g_pD3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
    g_pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
    g_pD3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
    g_pD3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
    g_pD3DDevice->SetRenderState(D3DRS_ZENABLE, iZEnable);
	g_pD3DDevice->SetRenderState(D3DRS_POINTSIZE, *((DWORD*)&fBatchDot3DSize));
    g_pD3DDevice->SetViewport(&g_pViewPort);
    g_pD3DDevice->SetFVF(D3DFVF_MY_VERTEX);
    g_pD3DDevice->DrawPrimitiveUP(D3DPT_POINTLIST, iBatchDot3DPos, g_BatchDot3DList, sizeof(Vertex));

	if (iClear)
	{
		delete[] g_BatchDot3DList;
		g_BatchDot3DList = NULL;
		iBatchDot3DMax = 0;
	}

	iBatchDot3DPos = 0;
}

// Initialize 3D line batch
void SetBatchSizeLine3D(int iSize)
{
	// If we have a batch, kill it
	if (g_BatchLine3DList)
		delete[] g_BatchLine3DList;

	// Create new batch
	g_BatchLine3DList = new (nothrow) Vertex[iSize * 2];

	if (g_BatchLine3DList)
	{
		// Batch creation succeeded
		iBatchLine3DPos = 0;
		iBatchLine3DMax = iSize;
	} else {
		// Batch creation failed: Make sure we can never add to it
		iBatchLine3DPos = 0;
		iBatchLine3DMax = 0;
	}
}

// Add 3D line to batch
void AddBatchLine3D(float fX1, float fY1, float fZ1, float fX2, float fY2, float fZ2)
{
	// If we haven't reached the batch limit
	if (iBatchDot3DPos < iBatchDot3DMax)
	{
		// Add line
		g_BatchDot3DList[(iBatchDot3DPos * 2)].x = fX1;
		g_BatchDot3DList[(iBatchDot3DPos * 2)].y = fY1;
		g_BatchDot3DList[(iBatchDot3DPos * 2)].z = fZ1;
		g_BatchDot3DList[(iBatchDot3DPos * 2)].color = g_Forecolour;

		g_BatchDot3DList[(iBatchDot3DPos * 2) + 1].x = fX2;
		g_BatchDot3DList[(iBatchDot3DPos * 2) + 1].y = fY2;
		g_BatchDot3DList[(iBatchDot3DPos * 2) + 1].z = fZ2;
		g_BatchDot3DList[(iBatchDot3DPos * 2) + 1].color = g_Forecolour;

		iBatchDot3DPos++;
	}
}

// Add 3D line to batch with overriding colour
void AddBatchLine3DC(float fX1, float fY1, float fZ1, float fX2, float fY2, float fZ2, DWORD dwColour)
{
	// If we haven't reached the batch limit
	if (iBatchLine3DPos < iBatchLine3DMax)
	{
		// Add line
		g_BatchLine3DList[(iBatchLine3DPos * 2)].x = fX1;
		g_BatchLine3DList[(iBatchLine3DPos * 2)].y = fY1;
		g_BatchLine3DList[(iBatchLine3DPos * 2)].z = fZ1;
		g_BatchLine3DList[(iBatchLine3DPos * 2)].color = dwColour;

		g_BatchLine3DList[(iBatchLine3DPos * 2) + 1].x = fX2;
		g_BatchLine3DList[(iBatchLine3DPos * 2) + 1].y = fY2;
		g_BatchLine3DList[(iBatchLine3DPos * 2) + 1].z = fZ2;
		g_BatchLine3DList[(iBatchLine3DPos * 2) + 1].color = dwColour;

		iBatchLine3DPos++;
	}
}

// Add 3D line to batch with overriding colour
void AddBatchLine3DGC(float fX1, float fY1, float fZ1, float fX2, float fY2, float fZ2, DWORD dwColour1, DWORD dwColour2)
{
	// If we haven't reached the batch limit
	if (iBatchLine3DPos < iBatchLine3DMax)
	{
		// Add line
		g_BatchLine3DList[(iBatchLine3DPos * 2)].x = fX1;
		g_BatchLine3DList[(iBatchLine3DPos * 2)].y = fY1;
		g_BatchLine3DList[(iBatchLine3DPos * 2)].z = fZ1;
		g_BatchLine3DList[(iBatchLine3DPos * 2)].color = dwColour1;

		g_BatchLine3DList[(iBatchLine3DPos * 2) + 1].x = fX2;
		g_BatchLine3DList[(iBatchLine3DPos * 2) + 1].y = fY2;
		g_BatchLine3DList[(iBatchLine3DPos * 2) + 1].z = fZ2;
		g_BatchLine3DList[(iBatchLine3DPos * 2) + 1].color = dwColour2;

		iBatchLine3DPos++;
	}
}

// Draw 3D line batch
void BatchDrawLine3D(int iCamera, int iZEnable, int iClear)
{
	DWORD dwCameraInternalData = GetInternalData((DWORD)iCamera);
	if (!dwCameraInternalData)
	{
		// Invalid camera
		*(DWORD*)g_pGlob->g_pErrorHandlerRef = (DWORD)7203;
		return;
	}

	g_pViewPort.X = *(int*)(dwCameraInternalData+200);
    g_pViewPort.Y = *(int*)(dwCameraInternalData+204);
    g_pViewPort.Width = *(int*)(dwCameraInternalData+208);
    g_pViewPort.Height = *(int*)(dwCameraInternalData+212);
	// TODO: Should these be CameraNear/Far?
    g_pViewPort.MinZ = 0.0f;
    g_pViewPort.MaxZ = 1.0f;

	D3DXMATRIX matWorld;
	D3DXMatrixIdentity(&matWorld);

	D3DXMATRIX matProj = *(D3DXMATRIX*)(dwCameraInternalData);
    D3DXMATRIX matView = *(D3DXMATRIX*)(dwCameraInternalData+64);

	g_pD3DDevice->SetTransform(D3DTS_WORLD, &matWorld);
    g_pD3DDevice->SetTransform(D3DTS_VIEW, &matView);
    g_pD3DDevice->SetTransform(D3DTS_PROJECTION, &matProj);
    g_pD3DDevice->SetTexture(0, NULL);
    g_pD3DDevice->SetTexture(1, NULL);
    g_pD3DDevice->SetVertexShader(NULL);
	g_pD3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
    g_pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
    g_pD3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
    g_pD3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
    g_pD3DDevice->SetRenderState(D3DRS_ZENABLE, iZEnable);
    g_pD3DDevice->SetViewport(&g_pViewPort);
    g_pD3DDevice->SetFVF(D3DFVF_MY_VERTEX);

	g_pD3DDevice->DrawPrimitiveUP(D3DPT_LINELIST, iBatchLine3DPos, g_BatchLine3DList, sizeof(Vertex));

	if (iClear)
	{
		delete[] g_BatchLine3DList;
		g_BatchLine3DList = NULL;
		iBatchLine3DMax = 0;
	}

	iBatchLine3DPos = 0;
}

// The Utah Teapot
void MakeObjectTeapot(int iID)
{
    if (FAILED(D3DXCreateTeapot(g_pD3DDevice, &g_pMesh3D, NULL)))
        return;

    MakeObjectFromDXMesh(iID, g_pMesh3D);
}

// Polygon object
void MakeObjectPolygon(int iID, int iSides)
{
	if (iSides < 3)
		return;

	if (FAILED(D3DXCreatePolygon(g_pD3DDevice, 1.0f, iSides, &g_pMesh3D, NULL)))
		return;

	MakeObjectFromDXMesh(iID, g_pMesh3D);
}

// Hey! Look! It's another function with a confusing name
void MakeObjectTorus(int iID, float fInnerRadius, float fOuterRadius, int iSides, int iRings)
{
	if ((fInnerRadius < 0.0f) || (fOuterRadius < 0.0f) || (iSides < 3) || (iRings < 3) )
		return;

     if (FAILED(D3DXCreateTorus(g_pD3DDevice, fInnerRadius, fOuterRadius, iSides, iRings, &g_pMesh3D, NULL)))
		 return;

	 MakeObjectFromDXMesh(iID, g_pMesh3D);
}

// Pyramid thingy
void MakeObjectPyramid(int iID, float fX, float fY, float fZ)
{
	sVertex VertexList[] =	{
								{ -0.5f, -0.5f,  0.5f,  0.000000f,  0.447214f,  0.894427f, 0.0f, 1.0f },
								{  0.5f, -0.5f,  0.5f,  0.000000f,  0.447214f,  0.894427f, 1.0f, 1.0f },
								{  0.0f,  0.5f,  0.0f,  0.000000f,  0.447214f,  0.894427f, 0.5f, 0.0f },
								{  0.5f, -0.5f,  0.5f,  0.894427f,  0.447214f,  0.000000f, 0.0f, 1.0f },
								{  0.5f, -0.5f, -0.5f,  0.894427f,  0.447214f,  0.000000f, 1.0f, 1.0f },
								{  0.0f,  0.5f,  0.0f,  0.894427f,  0.447214f,  0.000000f, 0.5f, 0.0f },
								{  0.5f, -0.5f, -0.5f,  0.000000f,  0.447214f, -0.894427f, 1.0f, 1.0f },
								{ -0.5f, -0.5f, -0.5f,  0.000000f,  0.447214f, -0.894427f, 0.0f, 1.0f },
								{  0.0f,  0.5f,  0.0f,  0.000000f,  0.447214f, -0.894427f, 0.5f, 0.0f },
								{ -0.5f, -0.5f, -0.5f, -0.894427f,  0.447214f,  0.000000f, 1.0f, 1.0f },
								{ -0.5f, -0.5f,  0.5f, -0.894427f,  0.447214f,  0.000000f, 0.0f, 1.0f },
								{  0.0f,  0.5f,  0.0f, -0.894427f,  0.447214f,  0.000000f, 0.5f, 0.0f },
								{ -0.5f, -0.5f,  0.5f,  0.000000f, -1.000000f,  0.000000f, 0.0f, 1.0f },
								{ -0.5f, -0.5f, -0.5f,  0.000000f, -1.000000f,  0.000000f, 0.0f, 0.0f },
								{  0.5f, -0.5f,  0.5f,  0.000000f, -1.000000f,  0.000000f, 1.0f, 1.0f },
								{ -0.5f, -0.5f, -0.5f,  0.000000f, -1.000000f,  0.000000f, 0.0f, 0.0f },
								{  0.5f, -0.5f, -0.5f,  0.000000f, -1.000000f,  0.000000f, 1.0f, 0.0f },
								{  0.5f, -0.5f,  0.5f,  0.000000f, -1.000000f,  0.000000f, 1.0f, 1.0f }
							};

	short IndexList[] =		{
								{  0 },
								{  1 },
								{  2 },
								{  3 },
								{  4 },
								{  5 },
								{  6 },
								{  7 },
								{  8 },
								{  9 },
								{ 10 },
								{ 11 },
								{ 12 },
								{ 13 },
								{ 14 },
								{ 15 },
								{ 16 },
								{ 17 }
							};

    sVertex		*pVertices	= NULL;
    ID3DXMesh	*pMeshNew	= NULL;
    short		*pIndices	= NULL;

	for (int i = 0; i < 18; i++)
	{
		VertexList[i].x *= fX;
		VertexList[i].y *= fY;
		VertexList[i].z *= fZ;
	}

    HRESULT HR = D3DXCreateMeshFVF(6, 18, D3DXMESH_MANAGED, D3DFVF_DBPMESH, g_pD3DDevice, &g_pMesh3D);

    HR = g_pMesh3D->LockVertexBuffer(D3DUSAGE_DYNAMIC, (LPVOID*)&pVertices);
    memcpy(pVertices, VertexList, sizeof(VertexList));
    g_pMesh3D->UnlockVertexBuffer();

    HR = g_pMesh3D->LockIndexBuffer(D3DUSAGE_DYNAMIC, (LPVOID*)&pIndices);
    memcpy(pIndices, IndexList, sizeof(IndexList));
    g_pMesh3D->UnlockIndexBuffer();

    MakeObjectFromDXMesh(iID, g_pMesh3D);
}